@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008 Johannes Reinhardt <jreinhardt@ist-dein-freund.de>
-# Copyright 2012-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2012-2020 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This modules handles export of LaTeX Code'''
 
@@ -21,7 +20,7 @@ logger = logging.getLogger('zim.formats.latex')
 
 info = {
 	'name': 'latex',
-	'desc':	'LaTeX',
+	'desc': 'LaTeX',
 	'mimetype': 'application/x-tex',
 	'extension': 'tex',
 	'native': False,
@@ -31,7 +30,7 @@ info = {
 }
 
 
-encode_re = re.compile(r'(\&|\$|\^|\%|\#|\_|\\|\<|\>|\n)')
+encode_re = re.compile(r'([&$^%#_\\<>\n])')
 encode_dict = {
 	'\\': '$\\backslash$',
 	'&': '\\$',
@@ -49,49 +48,49 @@ encode_dict = {
 class Dumper(TextDumper):
 
 	BULLETS = {
-		UNCHECKED_BOX : '\\item[\\Square]',
-		XCHECKED_BOX  : '\\item[\\XBox]',
-		CHECKED_BOX   : '\\item[\\CheckedBox]',
-		MIGRATED_BOX  : '\\item[\\RIGHTarrow]',
-		BULLET        : '\\item',
+		UNCHECKED_BOX: '\\item[\\Square]',
+		XCHECKED_BOX: '\\item[\\XBox]',
+		CHECKED_BOX: '\\item[\\CheckedBox]',
+		MIGRATED_BOX: '\\item[\\RIGHTarrow]',
+		BULLET: '\\item',
 	}
 
 	SECTIONING = {
 		'report': {
-			1:'\\chapter{%s}',
-			2:'\\section{%s}',
-			3:'\\subsection{%s}',
-			4:'\\subsubsection{%s}',
-			5:'\\paragraph{%s}'
+			1: '\\chapter{%s}',
+			2: '\\section{%s}',
+			3: '\\subsection{%s}',
+			4: '\\subsubsection{%s}',
+			5: '\\paragraph{%s}'
 		},
 		'article': {
-			1:'\\section{%s}',
-			2:'\\subsection{%s}',
-			3:'\\subsubsection{%s}',
-			4:'\\paragraph{%s}',
-			5:'\\subparagraph{%s}'
+			1: '\\section{%s}',
+			2: '\\subsection{%s}',
+			3: '\\subsubsection{%s}',
+			4: '\\paragraph{%s}',
+			5: '\\subparagraph{%s}'
 		},
 		'book': {
-			1:'\\part{%s}',
-			2:'\\chapter{%s}',
-			3:'\\section{%s}',
-			4:'\\subsection{%s}',
-			5:'\\subsubsection{%s}'
+			1: '\\part{%s}',
+			2: '\\chapter{%s}',
+			3: '\\section{%s}',
+			4: '\\subsection{%s}',
+			5: '\\subsubsection{%s}'
 		}
 	}
 
 	TAGS = {
-		EMPHASIS:		('\\emph{', '}'),
-		STRONG:			('\\textbf{', '}'),
-		MARK:			('\\uline{', '}'),
-		STRIKE:			('\\sout{', '}'),
-		TAG:			('', ''), # No additional annotation (apart from the visible @)
-		SUBSCRIPT:		('$_{', '}$'),
-		SUPERSCRIPT:	('$^{', '}$'),
+		EMPHASIS: ('\\emph{', '}'),
+		STRONG: ('\\textbf{', '}'),
+		MARK: ('\\uline{', '}'),
+		STRIKE: ('\\sout{', '}'),
+		TAG: ('', ''), # No additional annotation (apart from the visible @)
+		SUBSCRIPT: ('$_{', '}$'),
+		SUPERSCRIPT: ('$^{', '}$'),
 	}
 
 	TEMPLATE_OPTIONS = {
-		'document_type': Choice('report', ('report', 'article','book'))
+		'document_type': Choice('report', ('report', 'article', 'book'))
 	}
 
 	def dump(self, tree):
@@ -110,7 +109,7 @@ class Dumper(TextDumper):
 
 	def dump_pre(self, tag, attrib, strings):
 		indent = int(attrib.get('indent', 0))
-		text = u''.join(strings)
+		text = ''.join(strings)
 		text = text.replace('\n\n', '\n') # remove newlines introduces by encode_text
 		strings = text.splitlines(True)
 		if indent:
@@ -122,10 +121,12 @@ class Dumper(TextDumper):
 
 	def dump_h(self, tag, attrib, strings):
 		level = int(attrib['level'])
-		if level < 1: level = 1
-		elif level > 5: level = 5
+		if level < 1:
+			level = 1
+		elif level > 5:
+			level = 5
 
-		text = u''.join(strings)
+		text = ''.join(strings)
 		return [self.SECTIONING[self.document_type][level] % text]
 
 	def dump_ul(self, tag, attrib, strings):
@@ -136,21 +137,19 @@ class Dumper(TextDumper):
 
 	def dump_ol(self, tag, attrib, strings):
 		start = attrib.get('start', 1)
-		if os.name == 'nt':
-			start = start.encode('utf-8') # Weird locale dependent behavior
-		if start in string.lowercase:
+		if start in string.ascii_lowercase:
 			type = 'a'
-			start = string.lowercase.index(start) + 1
-		elif start in string.uppercase:
+			start = string.ascii_lowercase.index(start) + 1
+		elif start in string.ascii_uppercase:
 			type = 'A'
-			start = string.uppercase.index(start) + 1
+			start = string.ascii_uppercase.index(start) + 1
 		else:
 			type = '1'
 			start = int(start)
 
 		strings.insert(0, '\\begin{enumerate}[%s]\n' % type)
 		if start > 1:
-			strings.insert(1, '\setcounter{enumi}{%i}\n' % (start-1))
+			strings.insert(1, '\setcounter{enumi}{%i}\n' % (start - 1))
 		strings.append('\\end{enumerate}\n')
 
 		return TextDumper.dump_ol(self, tag, attrib, strings)
@@ -171,41 +170,38 @@ class Dumper(TextDumper):
 
 		return (bullet, ' ') + tuple(strings) + ('\n',)
 
+	def is_supported_image(self, path):
+		# Latex only supports limited image formats by default
+		# Whitelist pdf, png, jpg & eps -- all else should become a link
+		if '.' in path:
+			_, ext = path.rsplit('.', 1)
+			return ext.lower() in ('png', 'jpg', 'jpeg', 'eps', 'pdf')
+		else:
+			return False
 
 	def dump_img(self, tag, attrib, strings=None):
+		imagepath = self.linker.img(attrib['src'])
+		if not self.is_supported_image(imagepath):
+			attrib.setdefault('href', attrib['src'])
+			return self.dump_link(tag, attrib, strings)
+
 		# We try to get images about the same visual size,
 		# therefore need to specify dot density 96 dpi seems to be
 		# common for computer monitors
 		dpi = 96
 
-		if attrib.get('type') == 'equation':
-			try:
-				# Try to find the source, otherwise fall back to image
-				src = attrib['src'][:-4] + '.tex'
-				file = self.linker.resolve_source_file(src)
-				if file is not None:
-					equation = file.read().strip()
-				else:
-					equation = None
-			except FileNotFoundError:
-				logger.warn('Could not find latex equation: %s', src)
-			else:
-				if equation:
-					return ['\\begin{math}\n', equation, '\n\\end{math}']
-
 		if 'width' in attrib and not 'height' in attrib:
 			options = 'width=%fin, keepaspectratio=true' \
-					% ( float(attrib['width']) / dpi )
+					% (float(attrib['width']) / dpi)
 		elif 'height' in attrib and not 'width' in attrib:
 			options = 'height=%fin, keepaspectratio=true' \
-					% ( float(attrib['height']) / dpi )
+					% (float(attrib['height']) / dpi)
 		elif 'height' in attrib and 'width' in attrib:
 			options = 'height=%fin, width=%fin' \
-					% ( float(attrib['height']) / dpi, float(attrib['width']) / dpi )
+					% (float(attrib['height']) / dpi, float(attrib['width']) / dpi)
 		else:
 			options = ''
 
-		imagepath = self.linker.img(attrib['src'])
 		if imagepath.startswith('file://'):
 			imagepath = File(imagepath).path # avoid URIs here
 		image = '\\includegraphics[%s]{%s}' % (options, imagepath)
@@ -220,7 +216,7 @@ class Dumper(TextDumper):
 		href = self.linker.link(attrib['href'])
 		href = url_encode(href, URL_ENCODE_READABLE)
 		if strings:
-			text = u''.join(strings)
+			text = ''.join(strings)
 		else:
 			text = href
 		return ['\\href{%s}{%s}' % (href, text)]
@@ -228,10 +224,10 @@ class Dumper(TextDumper):
 	def dump_code(self, tag, attrib, strings):
 		# Here we try several possible delimiters for the inline verb
 		# command of LaTeX
-		text = u''.join(strings)
+		text = ''.join(strings)
 		for delim in '+*|$&%!-_':
 			if not delim in text:
-				return ['\\lstinline'+delim+text+delim]
+				return ['\\lstinline' + delim + text + delim]
 		else:
 			assert False, 'Found no suitable delimiter for verbatim text: %s' % element
 
@@ -243,7 +239,7 @@ class Dumper(TextDumper):
 
 		aligns, _wraps = TableParser.get_options(attrib)
 		rowline = lambda row: '&'.join([' ' + cell + ' ' for cell in row]) + '\\tabularnewline\n\hline'
-		aligns = map(lambda a: 'l' if a == 'left' else 'r' if a == 'right' else 'c' if a == 'center' else 'l', aligns)
+		aligns = ['l' if a == 'left' else 'r' if a == 'right' else 'c' if a == 'center' else 'l' for a in aligns]
 
 		for i, row in enumerate(rows):
 			for j, (cell, align) in enumerate(zip(row, aligns)):
@@ -259,7 +255,7 @@ class Dumper(TextDumper):
 		table += [rowline(row) for row in rows[1:]]
 
 		table.append('\end{tabular}')
-		return map(lambda line: line+"\n", table)
+		return [line + "\n" for line in table]
 
 	def dump_line(self, tag, attrib, strings=None):
 		return '\n\\hrule\n'
